package fr.natan.cleanarchitectureaddressservice.infrastructure.ports.input.controller;

import fr.natan.cleanarchitectureaddressservice.domain.entity.Address;
import fr.natan.cleanarchitectureaddressservice.domain.exceptions.*;
import fr.natan.cleanarchitectureaddressservice.domain.ports.input.AddressInputService;
import fr.natan.cleanarchitectureaddressservice.infrastructure.ports.input.feignclient.entity.model.EmployeeModel;
import fr.natan.cleanarchitectureaddressservice.infrastructure.ports.input.feignclient.service.EmployeeService;
import fr.natan.cleanarchitectureaddressservice.infrastructure.ports.output.address_models.AddressDto;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Optional;

@RestController
public class
AddressController {
    private final AddressInputService addressInputService;
    private final EmployeeService employeeService;

    public AddressController(AddressInputService addressInputService, EmployeeService employeeService) {
        this.addressInputService = addressInputService;
        this.employeeService = employeeService;
    }

    @RequestMapping(value = "/addresses", method = RequestMethod.GET)
    private List<Address> getAllAddresses(){
        return addressInputService.getAllAddresses();
    }

    @RequestMapping(value = "/addresses", method = RequestMethod.POST)
    private Address createAddress(@RequestBody AddressDto addressDto) throws AddressFieldsEmptyException, AddressAlreadyExistsException, AddressPBInvalidException, AddressNumInvalidException {
        return addressInputService.createAddress(addressDto);
    }

    @RequestMapping(value = "/addresses/{addressID}", method = RequestMethod.DELETE)
    private void deleteAddress(@PathVariable(name = "addressID") Long addressID) throws AddressNotFoundException, AddressAssignedEmployeesException {
        addressInputService.deleteAddress(addressID);
    }

    @RequestMapping(value = "/addresses/{addressID}", method = RequestMethod.PUT)
    private Address updateAddress(@PathVariable(name = "addressID") Long addressID, @RequestBody AddressDto addressDto)
            throws AddressFieldsEmptyException, AddressNotFoundException, AddressPBInvalidException, AddressNumInvalidException,
            AddressAlreadyExistsException {
        return addressInputService.updateAddress(addressID, addressDto);
    }

    @RequestMapping(value = "/addresses/{addressID}", method = RequestMethod.GET)
    private Optional<Address> getAddress(@PathVariable(name = "addressID") Long addressID) throws AddressNotFoundException {
        return addressInputService.getAddressByID(addressID);
    }
    @RequestMapping(value = "/employees/addresses/{addressID}", method = RequestMethod.GET)
    private  List<EmployeeModel> employeesAtAddressThis(@PathVariable(name = "addressID") Long addressID) throws AddressNotFoundException {
        return addressInputService.getEmployeesLivingAtAddressThis(addressID);
    }
}
